@@ -8,3 +8,5 @@ Y ~ X + Z + X:Z
 '
 
 est1 <- modsem(m1, data = oneInt, convergence = 1e-2, method = "qml")
+print(summary(est1))
+plot_interaction(x = "X", z = "Z", y = "Y", xz = "X:Z", vals_z = c(-0.5, 0.5), model = est1)
